/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
/**
 * @file config.js
 * @brief Provides functionality used for reading and writing the
 * configuration. For setting an application to open certain file types with,
 * the key of the config item should be the file extension INCLUDING the
 * prefixed dot (.).
 * @author Michiel Westerink
 */

//

module.exports = {
    getConfigItem,
    changeConfig,
    initConfig,
    writeConfigToFile,
    readConfigFile,
	readDefaultConfig,
    getDefaultConfigFileLocation,
    getDefaultLogFileLocation,
    getlsofExtentions
}

const fs = require('fs');
const jsonfile = require('jsonfile');
const utils = require('./utils');
var log = require('electron-log');
const util= require('util');

// geeft exception als de OpenImsTransferAgent folder niet bestaat
//log.info("===============Start Proton TA ");



/*
	logging wil use config to determine the log directory loglevel etc.
	so we need to include (require) logging after these routines are available (module.exports)
*/

const logging = require('./logging');

function getlsofExtentions() {
	logging.logWarning('getlsofExtentions start');
	var lsofExtentions = getConfigItem("lsofExtensions");
	//alert("getlsofExtentions");
	return lsofExtentions;
}

function getDefaultlsofExtentions() {
    var defaultlsofExtentions = ['doc', 'docx', 'docm', 'xls', 'xlsx', 'xlsm', 'ppt', 'pptx', 'pptm', 'pps', 'ppsm', 'rtf'];
	return defaultlsofExtentions;
}

function getDefaultConfigFileLocation() {
	return utils.getApplicationDirectory() + 'settings.json';
	//return utils.getApplicationDirectory() + 'config.json';
}

function getDefaultLogFileLocation() {
	return utils.getApplicationDirectory() + 'log.txt';
}

/**
 * Initialize the config. If there is no config file present on the default
 * location, create the config file and fill it with default values
 */
function initConfig()
{
    var defaultConfig = writeDefaultConfig();
    if (defaultConfig) {
//        log.transports.file.file = defaultConfig["logFileLocation"];
    }

    try {
        fs.statsSync(getDefaultConfigFileLocation())
    } catch (e) {
        if(e.toString().indexOf("ENOENT") !== -1) {
            writeDefaultConfig();
            logging.logWarning('initConfig writeDefaultConfig');
        }
    }
}

/**
 * If no configuration file is present, write a new configuration file. Some
 * platforms might have their own quirks or settings, so the function calls
 * a different function for each supported platform.
 */

function readDefaultConfig() {
    var defaultConfig=null;
    switch(utils.detectPlatform()) {
        case "darwin": //OSX
            defaultConfig = getDefaultConfigMac();
            break;
        case "linux": //Linux
            defaultConfig = getDefaultConfigLinux();
           break;
        case "win32": //Windows
            defaultConfig = getDefaultConfigWindows();
            break;
        default:
            break;
    }
	return defaultConfig;
}

function writeDefaultConfig()
{
    var defaultConfig = readDefaultConfig();
    if (defaultConfig) {
		//logging.logWarning('writing DefaultConfig');
        writeConfigToFile(defaultConfig);
    }
    return defaultConfig;
}

/**
 * Default options for Mac clients
 *
 * @return array containing various settings
 */
function getDefaultConfigMac()
{

/*
    var defaultConfig = {
        //Default location for the log file
        "logFileLocation":getDefaultLogFileLocation(),
        //If true, logs are written to a file
        "loggingEnabled":true,
        //If true, user can take over lock
        "forceLockEnabled":true,
        //Set the amount of time (in ms) the TransferAgent should wait before
        //the document is opened.
        "waitForApplication":"8",
        "lsofExtensions":getDefaultlsofExtentions()
    };
*/

    var defaultConfig = {
		/* // loggingEnabled can be
			<select id="loggingEnabled" name="loggingEnabled">
				<option value="1">Debug</option>
				<option value="2">Uitgebreid</option>
				<option value="3">Info</option>
				<option value="4">waarschuwing</option>
				<option value="5">Fouten</option>
				<option value="10">Uit</option>
			</select>
		*/

        //Default location for the log file
        "logFileLocation":getDefaultLogFileLocation(),
        //If > 0, logs are written to a file
        "loggingEnabled":"4",

        // "debuggingEnabled":"0",

        //If true, user can take over lock
        "forceLockEnabled":"1", //waarschuwing
        //Set the amount of time (in ms) the TransferAgent should wait before
        //the document is opened.
        "waitForApplication":"8",
        // 20210212 KVD Max size in MB
        "maxFileSizeInMegabytes" : "191",
        "lsofExtensions":getDefaultlsofExtentions()
    };

    return defaultConfig;
}

/**
 * Default options for Linux clients
 *
 * @return array containing various settings
 */
function getDefaultConfigLinux()
{
    var defaultConfig = {
        //Default location for the log file
        "logFileLocation":getDefaultLogFileLocation(),
        //If true, logs are written to a file
        "loggingEnabled":true,
        //If true, user can take over lock
        //"debuggingEnabled":"0",//
        "forceLockEnabled":false,
        //Set the amount of time (in ms) the TransferAgent should wait before
        //the document is opened.
        "waitForApplication":"8",
        "lsofExtensions":getDefaultlsofExtentions()
    };
    return defaultConfig;
}

/**
 * Default options for Windows clients
 *
 * @return array containing various settings
 */
function getDefaultConfigWindows()
{
    var defaultConfig = {
        //Default location for the log file
        "logFileLocation":getDefaultLogFileLocation(),
        //If true, logs are written to a file
        "loggingEnabled":true,
        //"debuggingEnabled":"0",
        //If true, user can take over lock
        "forceLockEnabled":false,
        //Set the amount of time (in ms) the TransferAgent should wait before
        //the document is opened.
        "waitForApplication":"8",
        "lsofExtensions":getDefaultlsofExtentions()
    };
    return defaultConfig;
}

/**
 * Get item from config file
 *
 * @param string key -  Key of the item to retrieve
 * @return string - Value of the item
 */
function getConfigItem(key)
{
    var configuration = readConfigFile();
    // This line CRASHES (in log.info) when no settings.json is available
//    logging.logInfo("getConfigItem ["+ key + "] = typeof "+typeof(configuration[key]));

    return configuration[key];
}


/**
 * Write configuration to a file, whilst formatting it so it is easily
 * readable.
 *
 * @param JSON object - JSON object containing the configuration.
 */
function writeConfigToFile(configuration)
{
//    log.info(configuration);
    jsonfile.writeFileSync(
        getDefaultConfigFileLocation(),
        configuration,
        {spaces: 4});
}

/**
 * Reads the configuration file and returns it as a JSON object
 *
 * @return string - Configuration as JSON object
 */
function readConfigFile()
{
//    return readDefaultConfig(); //!!
    configFilePath = getDefaultConfigFileLocation();
 	try {
        fs.statSync(utils.getApplicationDirectory());
    } catch (e) {
        if(e.toString().indexOf("ENOENT") !== -1) {
            fs.mkdirSync(utils.getApplicationDirectory());
            initConfig();
            logging.logWarning('readConfigFile initConfig '+ utils.getApplicationDirectory());
        }
    }

    try {
		fs.statSync(configFilePath);
    } catch (e) {
        if(e.toString().indexOf("ENOENT") !== -1) {
            initConfig();
            logging.logWarning('readConfigFile initConfig ' + configFilePath);
        }
    }
    try {
        return jsonfile.readFileSync(configFilePath);
    } catch (e) {
        return readDefaultConfig();  //!!
    }
}

/**
 * Change or add an option to the config file. If the key already exists, the
 * new value will overwrite the current value. If the key doesn't exist yet,
 * a new key is added with the given value.
 *
 * @param string - key Key of the setting
 * @param string - value Value of the setting
 */
function changeConfig(key, value)
{
    //First read the current configuration
    var config = readConfigFile();

	//logging.logDebug("changeConfig ["+ key + "] = typeof "+typeof(config[key]) + ", newval=" + value + ", val="+config[key]);

    if (settingExists(key)) {
        config[key] = value;
    } else {
        //Create new object to push to JSON array
        config[key] = value;
//        var obj = {};
//        obj[key] = value;
//        config.push(obj);
    }

    //Add the new entry
    writeConfigToFile(config);
}

/**
 * Check whether or not a setting has already been defined in settings.json
 *
 * @return bool - True if defined, false if not
 */
function settingExists(key)
{
    configuration = readConfigFile();
    for (var i = 0; i < configuration.length; i++) {
        if (Object.keys(configuration[i])[0] === key) {
            return true;
        }
    }
    return false;
}
