/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
 /**
 * @file openfile.js
 * @brief Provides functions related to opening files.
 * @author Michiel Westerink
 */
const {BrowserWindow} = require('electron');
const {app} = require('electron');
const {dialog} = require('electron');

const child_process = require('child_process');
const events = require('events');
const fs = require('fs');
const request = require('request');
const wget = require('wget-improved');

const config = require('./config');
const logging = require('./logging');
const upload = require('./uploadfile');
const utils = require('./utils');


module.exports = {
    openHTTPFile,
    openFile,
    lockFile,
    downloaded,
    verifyLink
}

var eventEmitter = new events.EventEmitter();

/**
 * Open a file. Only the path to the file has to be given, the platform
 * and which application to use are determined automagically
 *
 * @param IMSDoc imsDoc - IMSDocument object to open
 * @param bool different - True if file has been changed, false if not
 */
function openFile(imsDoc, different)
{
    logging.logVerbose('openFile: different: ' + different + '  readonly: ' + imsDoc.readonly);
    //logging.logVerbose('openFile: imsdoc: ' + utils.objectToString(imsDoc));
    //logging.logVerbose('openFile: process.env: ' + utils.objectToString(process.env));

    if(different) {
        var buttons = ['Afsluiten'];

        var dialogOptions = {
            'type' : 'info',
            'title' : 'Verifi&euml;ren mislukt',  //!! dialog aanpassen voor &xxx;
            'message' : 'Verificatie van het gedownloade bestand mislukt',
            'buttons' : buttons,
            'defaultID' : 0
        };

        logging.logVerbose('openFile: message: ' + dialogOptions.message);

        dialog.showMessageBox(dialogOptions,
            function callback(buttonIndex) {
                if (buttonIndex === 0) {
                    upload.unlockFile(imsDoc);
                }
        });
        logging.logVerbose('openFile: before return/quit at error');
        return;   // 20170804 KVD SD08674
    }

    if(imsDoc.readonly) {
        var buttons = ['OK'];

        var msg = utils.ML('Bestand is alleen lezen geopend, wijzigingen worden niet opgeslagen op de server.',
                           'File is opened read only, changes will not be saved on the server.');
        var dialogOptions = {
            'type' : 'warning',
            'title' : utils.ML('Alleen lezen', 'Read-only'),
            'message' : msg,
            'buttons' : buttons,
            'defaultID' : 0
        };

        logging.logVerbose('openFile (readonly): message: ' + dialogOptions.message);

        dialog.showMessageBox(dialogOptions);
    }

    //Should a new platform have to be added, simply look up which code
    //Node JS returns for said platform, add a function which handles
    //opening files on that platform and add a new case.
    switch(utils.detectPlatform()) {
        case "darwin": //macOS
           openFileMac(imsDoc.filePath);
           break;
        case "linux": //Linux
           openFileLinux(imsDoc.filePath);
           break;
        case "win32": //Windows
           openFileWindows(imsDoc.filePath);
           break;
        default:
           break;
    }

    //ericd 021216 show "Klik als u klaar bent" for other then specified lsof file extensions
    if(!utils.hasKnownLsofExt(imsDoc)) {
        utils.waitForMessageBox();

        if(!imsDoc.readonly) {
            utils.compareLocalWithRemote(upload.uploadFile, imsDoc);
        } else {
            utils.cleanUp(imsDoc);
        }

        return;
    }

    //Wait for a certain amount of time (configurable) for the application to
    //start.

/*
   logging.logDebug('openFile ===================== start LSOF stuff ================================');

   logging.logDebug('openFile setTimeout () config.getConfigItem(waitForApplication) = ' + (config.getConfigItem('waitForApplication')*1000));


	setTimeout(function() {
		 logging.logDebug('openFile eerste setTimeout 2 seconden na openFile start');

		 fileInUse(imsDoc);

		 logging.logDebug('openFile eerste setTimeout 2 seconden na openFile end');

	}, 2000 ); //(config.getConfigItem('waitForApplication')*1000)

*/

	utils.newCreateWindow();

    setTimeout(function() {
        // SD09285 gv 23-04-2018 (lsof very slow on OSX 10.13)
        // var intervalObject = setInterval(fileInUse, 5000, imsDoc);

        logging.logDebug('openFile in setTimeout setInterval  ' + (config.getConfigItem('waitForApplication')*1000));


        var intervalObject = setInterval(fileInUse, (config.getConfigItem('waitForApplication')*1000), imsDoc);

        eventEmitter.on('fileClosed', function() {

        	logging.logDebug('openFile in setTimeout in eventEmitter.on(fileClosed)');

        	//logging.logDebug('openFile in setTimeout in eventEmitter.on(fileClosed) intervalObject = ' + utils.objectToString(intervalObject));

        	//zz = intervalObject.ref();

        	//logging.logDebug('openFile in setTimeout in eventEmitter.on(fileClosed) zz = ' + utils.objectToString(zz));

        	//logging.logDebug('openFile in setTimeout in eventEmitter.on(fileClosed) imsDoc = ' + utils.objectToString(imsDoc));

            clearInterval(intervalObject);
            if(!imsDoc.readonly) {
                utils.compareLocalWithRemote(upload.uploadFile, imsDoc);
            } else {
                utils.cleanUp(imsDoc);
            }
        });
    }, (config.getConfigItem('waitForApplication')*1000));
}

function detectFileClosed(imsDoc) {

	logging.logDebug('detectFileClosed = ' + utils.objectToString(imsDoc));

	// SD09285 gv 23-04-2018 (lsof very slow on OSX 10.13)
	// var intervalObject = setInterval(fileInUse, 5000, imsDoc);
	var intervalObject = setInterval(fileInUse, (config.getConfigItem('waitForApplication')*1000), imsDoc);

    eventEmitter.on('fileClosed', function() {

    	logging.logDebug('detectFileClosed fileClosed  = ' + utils.objectToString(imsDoc));

        clearInterval(intervalObject);
        if(!imsDoc.readonly) {
            utils.compareLocalWithRemote(upload.uploadFile, imsDoc);
        } else {
            utils.cleanUp(imsDoc);
        }
    });
}

/**
 * Open a file on a macOS machine and monitors it for changes.
 *
 * @param string fileName - The file to open
 */
function openFileMac(fileName)
{

// https://nodejs.org/api/child_process.html

    //When the command 'open <path>' is executed, macOS automatically determines
    //which application should be used to open the file.
    command = 'open ' + fileName;

    logging.logVerbose('openFileMac: ' + command);

    //Open the the application and the specified file
    var open_child_process = child_process.exec (command,  function (error, stdout, stderr) {

        if(error) {
            logging.logError('openFileMac: ERROR: ' + error);
            logging.logError('openFileMac: stdout: ' + stdout);
            logging.logError('openFileMac: stderr: ' + stderr);
        }
    });

 /*
 	open_child_process.on('exit', function (code, signal) {
		logging.logDebug('openFileMac: open_child_process process exited code= ' + code + ' signal= '+signal);
	});

	open_child_process.on('close', function (code, signal) {
	  //console.log(`child process exited with code ${code}`);
	  logging.logDebug('openFileMac: open_child_process child process closed with code ' + code + ' signal= '+signal);
	});
*/

	// utils.objectToString(open_child_process)); will produce ~300 log lines
    // logging.logDebug('openFileMac: open_child_process: ' + utils.objectToString(open_child_process)); // open_child_process);
    // logging.logDebug('openFileMac: objectToString open_child_process: ' + utils.objectToString(open_child_process));

}

/**
 * Open a file on a Linux machine
 *
 * @param string fileName - The file to open
 */
function openFileLinux(fileName)
{
    command = 'xdg-open ' + fileName;

    //Open the the application and the specified file
    child_process.exec (command,  function (error, stdout, stderr) {
        logging.logVerbose('Exit: ' + error);
        logging.logVerbose('Output: ' + stdout);
        logging.logVerbose('Stderr: ' + stderr);
    });
}

/**
 * Open a file on a Windows machine
 *
 * @param string fileName - The file to open
 */
function openFileWindows(fileName)
{
}

/**
 * Download a file from a specified URL.
 *
 * @param IMSDocument imsDoc - IMSDoc to download
 * @param [optional] bool different - Should the file already be downloaded to
 * disk, it is checked to see if the file has been changed since the last time.
 * If it is, the file is downloaded again, if not, the local file is opened.
 */
function openHTTPFile(imsDoc, different)
{
    var IMSDoc = require('./document');
    var output = imsDoc.filePath;

//    console.log('openHTTPFile: stack: ' + utils.stacktrace());
//utils.showDialog("wacht ff...");
//    logging.logVerbose('openHTTPFile: imsDoc: ' + utils.objectToString(imsDoc));

    logging.logVerbose('openHTTPFile: different: ' + different + ', readonly: ' + imsDoc.readonly);
    //logging.logVerbose('openHTTPFile: readonly: ' + imsDoc.readonly);
    logging.logVerbose('openHTTPFile: httplink: ' + imsDoc.httpLink);
    //logging.logVerbose('openHTTPFile: imsDoc: ' + utils.objectToString(imsDoc));

    //If the file has been downloaded but its hash is the same of that of the
    //file on the server, just open the local file. If the MD5 hashes differ
    //from each other, the file is downloaded again.
    if(downloaded(imsDoc) && !different) {
        logging.logVerbose('openHTTPFile: not different, normal opening');
        openFile(imsDoc);

        return;
    } else if(!downloaded(imsDoc) && different) {
        try {
            fs.statSync(utils.getApplicationDirectory() + imsDoc.docDirectory);
        } catch (e) {
            if(e.toString().indexOf("ENOENT") !== -1) {
                fs.mkdirSync(utils.getApplicationDirectory() +
                    imsDoc.docDirectory);
            }
        }
    }

    //ericd 011216 use gate to download file
    var src = imsDoc.https + '://' + imsDoc.hostName
        + '/nkit/gate.php?sessionkey='
        + imsDoc.sessionKey
        +'&command=openims_get_blockcontent&downloadmethod=Koen&file='
        + imsDoc.imsSiteDirectory + imsDoc.fileName;

    var download = wget.download(src, output);

    download.on('error', function(err) {
            dialog.showErrorBox('Fout', JSON.stringify(err));
            logging.logError("openHTTPFile: ERROR: " + err);
            app.quit();
    });
    download.on('start', function(fileSize) {
            logging.logVerbose('openHTTPFile: fileSize: ' + fileSize);
    });
    download.on('end', function(output) {
            var tmpURL = imsDoc.httpLink;

            var filePath = utils.getApplicationDirectory() +
                imsDoc.docDirectory + "/" + imsDoc.fileName;

            utils.getDocumentMD5(imsDoc);
            utils.markCacheFile(imsDoc);
            utils.compareLocalWithRemote(openFile, imsDoc);
    });
}


function isLSOFRunning(imsDoc) {

	//logging.logDebug('isLSOFRunning: start');
	command = 'pgrep lsof ';

	//logging.logDebug('isLSOFRunning: command: ' + command);


		var isLSOFRunning_child_process = child_process.exec (command, function (error, stdout, stderr) {
			if(error) {
				//ericd no lsof, check one more time, in case of user saved, but didnt close file.

				//logging.logError('isLSOFRunning: ERROR: ' + error);
				//logging.logError('isLSOFRunning: ERROR stdout: ' + stdout);
				//logging.logError('isLSOFRunning: ERROR stderr: ' + stderr);

				var isLSOFRunningResult = stdout.trim();

				//logging.logDebug('isLSOFRunning: isLSOFRunningResult: LSOF = not Running !!!!!!!!!!!!!!!!!!!!!!!!' + isLSOFRunningResult);
				logging.logDebug('isLSOFRunning: LSOF is NOT Running');

				fileInUseUnix(imsDoc);

			} else {
				//logging.logDebug('isLSOFRunning: error: ' + error);
				//logging.logDebug('isLSOFRunning: stdout: ' + stdout);

				var isLSOFRunningResult = stdout.trim();

				//logging.logDebug('isLSOFRunning: isLSOFRunningResult: ' + isLSOFRunningResult);


				if (isLSOFRunningResult === "") {
					//logging.logDebug('isLSOFRunning: isLSOFRunningResult: empty string' + isLSOFRunningResult);

				} else {
					logging.logDebug('isLSOFRunning: LSOF is Running');
				}
				//logging.logDebug('isLSOFRunning: stderr: ' + stderr);
			}
		});
}


function lsofsaveFileInUseUnix(imsDoc) {

	//isLSOFRunning(imsDoc);
// hiermee wordt lsof altijd uitgevoerd als je het pid hebt

    var childPID = global.storage.lsofchildPID;
	//logging.logDebug("lsofsaveFileInUseUnix childPID = "+ childPID , false);

	if (!childPID) {
		isLSOFRunning(imsDoc);
	} else {
		fileInUseUnix(imsDoc);
	}


}


/**
 * Determine whether or not a file is in use by a program
 *
 * @param string filePath - Path to the file to examine
 * @return bool - True if file in use, false if not
 */
function fileInUse(imsDoc)
{
     switch(utils.detectPlatform()) {
         //Fall through, because OSX and Linux can use the same
         //function.
         case "darwin": //OSX
         case "linux": //Linux
         	lsofsaveFileInUseUnix(imsDoc);
            //fileInUseUnix(imsDoc);
            break;
         case "win32": //Windows
            break;
         default:
            break;
     }
}

/**
 * Should be called from fileInUse when the TransferAgent runs on a Unix system
 * which has the 'lsof' command implemented. Works at least on OSX and Linux
 * based operating systems.
 *
 * @param IMSDoc imsDoc - Document to check
 */




function fileInUseUnix(imsDoc)
{
    var filePath = utils.getApplicationDirectory() + imsDoc.docDirectory +
        "/" + imsDoc.fileName;

    var lsoffileIsOpen = 'n'+filePath;

    var childPID = global.storage.lsofchildPID;
	logging.logDebug("fileInUseUnix childPID = "+ childPID , false);

	if (!childPID) {

		//logging.logDebug('fileInUseUnix: not childPID : childPID = ' + childPID);

		//Compose the command with which the file is opened.
		// awk {if(NR>1) skip the header line
		// print $2 wil only output the PID
		command = 'lsof ' + filePath + ' |awk \'{if(NR>1)print $2}\'';
		logging.logDebug('fileInUseUnix: command: ' + command);

		//If the lsof command returns 0, a match has been found so the file is in
		//use by some process. If 1 is returned, no match has been found, meaning
		//the file isn't being used by a process.
		var firstlsof_child_process = child_process.exec (command, function (error, stdout, stderr) {
			if(error) {
				//ericd no lsof, check one more time, in case of user saved, but didnt close file.
				global.storage.lsofchildPID = null;
				eventEmitter.emit('fileClosed', imsDoc);
				//logging.logError('fileInUseUnix: ERROR: ' + error);
				//logging.logError('fileInUseUnix: ERROR stdout: ' + stdout);
				//logging.logError('fileInUseUnix: ERROR stderr: ' + stderr);
			} else {
				//logging.logDebug('fileInUseUnix: error: ' + error);
				//logging.logDebug('fileInUseUnix: stdout: ' + stdout);

				global.storage.lsofchildPID = stdout.trim();

				if (!global.storage.lsofchildPID) {

					//logging.logDebug('fileInUseUnix: awk file is open save the pid global.storage.lsofchildPID = false '+global.storage.lsofchildPID);

					global.storage.lsofchildPID = null;
					eventEmitter.emit('fileClosed', imsDoc);

				} else if (global.storage.lsofchildPID === "") {

					//logging.logDebug('fileInUseUnix: awk file is open save the pid global.storage.lsofchildPID  === "" '+global.storage.lsofchildPID);


					global.storage.lsofchildPID = null;
					eventEmitter.emit('fileClosed', imsDoc);

				}

				logging.logDebug('fileInUseUnix: global.storage.lsofchildPID: ' + global.storage.lsofchildPID);

				/*

				stdout zonder awk geeft;

				2019-09-19 17:54:48 fileInUseUnix: stdout: COMMAND PID  USER   FD   TYPE DEVICE SIZE/OFF     NODE NAME
				BBEdit  617 guido   16r   REG    1,4      462 15871500 /Users/guido/OpenImsTransferAgent/8a88a6ce4f28e28ce3989cfaf55dc3bf/scrap.txt

				*/

				//logging.logDebug('fileInUseUnix: stderr: ' + stderr);
			}
		});
/*
		firstlsof_child_process.on('exit', function (code, signal) {
		  logging.logDebug('firstlsof_child_process process exited ' + code + ' signal = '+signal);
		});
*/

	   //logging.logDebug('fileInUseUnix: firstlsof_child_process: ' + firstlsof_child_process);
	   // logging.logDebug('fileInUseUnix: objectToString firstlsof_child_process: ' + utils.objectToString(firstlsof_child_process));





	} else {
		//logging.logDebug('fileInUseUnix: got global.storage.lsofchildPID: ' + childPID);


		// lsof -F n -p <PID office> | grep <deel bestandsnaam>


		command = 'lsof -F n -p ' + global.storage.lsofchildPID  + " | grep " + filePath;
		logging.logDebug('fileInUseUnix: lsofchildPID command: ' + command);

		//If the lsof command returns 0, a match has been found so the file is in
		//use by some process. If 1 is returned, no match has been found, meaning
		//the file isn't being used by a process.
		var haspidlsof_child_process = child_process.exec (command, function (error, stdout, stderr) {
			if(error) {
				//ericd no lsof, check one more time, in case of user saved, but didnt close file.
				global.storage.lsofchildPID = null;
				eventEmitter.emit('fileClosed', imsDoc);
				//logging.logError('fileInUseUnix: lsofchildPID ERROR: ' + error);
				//logging.logError('fileInUseUnix: lsofchildPID ERROR stdout: ' + stdout);
				//logging.logError('fileInUseUnix: lsofchildPID ERROR stderr: ' + stderr);
			} else {


				//logging.logDebug('fileInUseUnix: lsofchildPID error: ' + error);

				var lsofresult = stdout.trim();
				//logging.logDebug('fileInUseUnix: (111) lsofchildPID stdout: ' + lsofresult);
				//logging.logDebug('fileInUseUnix: (111) lsof lsoffileIsOpen: ' + lsoffileIsOpen);


				if (lsofresult == lsoffileIsOpen) {
					//logging.logDebug('fileInUseUnix: lsoffileIsOpen : ' + lsofresult);
				} else {

					logging.logDebug('fileInUseUnix: lsof file Is Closed : ' + lsofresult);
					global.storage.lsofchildPID = null;
					eventEmitter.emit('fileClosed', imsDoc);
				}


				//logging.logDebug('fileInUseUnix: lsofchildPID stderr: ' + stderr);
			}
		});


	/*
		haspidlsof_child_process.on('exit', function (code, signal) {
			logging.logDebug('fileInUseUnix: haspidlsof_child_process process exited code= ' + code + ' signal= '+signal);

			//logging.logDebug('fileInUseUnix: haspidlsof_child_process process exited ERROR: ' + error);
		//	logging.logDebug('fileInUseUnix: haspidlsof_child_process process exited ERROR stdout: ' + stdout);
		//	logging.logDebug('fileInUseUnix: haspidlsof_child_process process exited ERROR stderr: ' + stderr);

		});

	*/
		//logging.logDebug('fileInUseUnix: haspidlsof_child_process: ' + haspidlsof_child_process);
	   // logging.logDebug('fileInUseUnix: objectToString haspidlsof_child_process: ' + utils.objectToString(haspidlsof_child_process));



	}

 }

/**
 * If an MS Office program has a file opened, a new file is created in the same
 * directory with the first two characters of the filename of the opened file
 * replaced by '~$' (without the quotes).
 *
 * So, for example, document.docx would get the lockfile ~$cument.docx. This
 * function checks if this file exists to determine whether or not the file
 * is being  used by an MS Office application
 *
 * @param string filePath - Path to the file to check
 */
function fileInUseOffice(filePath)
{
    logging.logVerbose('fileInUseOffice: interval');
}

/**
 * A document is downloaded to a directory which has the same name as the
 * directory on the OpenIMS server. If the directory exists and the document
 * is present in the directory, the file has already been downloaded.
 *
 * @param IMSDoc imsDocument - Instance of an IMS document
 * @return bool - True if downloaded, false if not
 */
function downloaded(imsDocument)
{
    try {
        fs.statSync(imsDocument.filePath);
    } catch (e) {
        if(e.toString().indexOf('ENOENT') !== -1) {
            return false;
        }
    }

    return true;
}

/**
 * If a document has already been downloaded, check if the previous upload was
 * successful by checking if a file 'cache' exists in the document
 * directory. If it does, the user should choose to use the local file or
 * the remote file.
 *
 * @param IMSDoc imsDocument - Document to check
 * @return bool - True if cached version should be used, false if remote
 * version should be used.
 */
function useCached(imsDocument)
{
    var useCachedDocument = true;
    var timestamp = '';

    var documentPath = utils.getApplicationDirectory();
    documentPath = documentPath + imsDocument.docDirectory + '/';
    documentPath = documentPath + 'cache';

    try {
        fs.statSync(documentPath);
    } catch (e) {
        if(e.toString().indexOf("ENOENT") !== -1) {
            return false;
        }
    }
    // 20210402 KVD SD12255 Gebruik lokaal bestand
    var message, buttons;
    if (config.getConfigItem('alwaysuselocalfile')) {
        message = utils.ML('Het lokale bestand is NIET verzonden, lokaal bestand wordt gebruikt.',
                               'Local file has NOT been uploaded, local file will be used.');
        buttons = [utils.ML('OK', 'OK')];
    } else {
///
        message = utils.ML('Wilt u het lokale bestand gebruiken of het bestand op de server ?\n\n' +
                               'Let op: Als u voor de serverversie kiest, wordt het lokale  bestand overschreven.',
                               'Do you want to use the local file or the file on the server ?\n\n' +
                               'Note: If you select the server version, the local version will be overwritten.' )
        buttons = [utils.ML('Lokale versie', 'Local version'), utils.ML('Serverversie', 'Server version')];
    }
    var dialogOptions = {
        'type' : 'info',
        'title' : utils.ML('Lokaal bestand gevonden', 'Local file found'),
        'message' : message,
        'buttons' : buttons,
        'defaultID' : 0
    };
    var buttonIndex = dialog.showMessageBox(dialogOptions);
        console.log(buttonIndex);

    if(buttonIndex === 0) {
        return true;  // local file
    } else if(buttonIndex === 1) {
        return  false;  // remote file
    }

}

/**
 * Try to lock the file for editing. If the file has been locked already, the
 * user is given the choice to quit the TransferAgent or to open the file in
 * read-only mode.
 *
 * Because of the asynchronous HTTP POST, the function openHTTPFile is called
 * from this function as well.
 *
 * @param IMSDoc imsDocument - Document that has to be locked
 * @param bool forceLock - True if lock should be forced, false if not
 */
function lockFile(imsDocument, forceLock)
{
    //logging.logVerbose('lockFile: trying to lock');

    var lockCommand = 'openims_try_to_lock_proton';  //ericd 060117 gate echo utf8 encoded user name

    logging.logVerbose('lockFile: commmand: ' + lockCommand);

    if(config.getConfigItem('forceLockEnabled') && forceLock) {
        logging.logVerbose('lockFile: forcing lock');
        lockCommand = 'openims_force_lock';
        logging.logVerbose('lockFile: commmand: ' + lockCommand);
    }

    var formData = {
        command : lockCommand,
        dir : imsDocument.imsSiteDirectory + imsDocument.fileName,
        sessionkey : imsDocument.sessionKey
    };

    request.post({url: imsDocument.https + '://' + imsDocument.hostName +
        '/nkit/gate.php',
        formData: formData}, function callback(err, response, body) {
            if(err) {
                var message = '';
                if(JSON.stringify(err).indexOf('SELF_SIGNED_CERT') !== -1) {
                    message = utils.ML('Certificaatfout','Certificate error');
                } else {
                    message = utils.ML('Onbekende fout','Unknown error');
                }
                dialog.showErrorBox(utils.ML('Fout','Error'), message);
                logging.logError('lockFile: ERROR ' + message);
                app.quit();
            }

            if(body !== undefined && body !== 'ok' && !forceLock) {
                var buttons = [utils.ML('Afsluiten','Close'), utils.ML('Alleen lezen', 'Read-only')];

				var doforceLock = (config.getConfigItem('forceLockEnabled') == "0");

				logging.logVerbose('Force doforceLock ' + doforceLock);

                if(doforceLock == true) {
                    buttons.push(utils.ML('Forceer', 'Force'));
                }

                logging.logVerbose('lockFile: body: ' + body);

                var message = body;

                if(body.indexOf("FATAL ERROR") !== -1) {
                    message = utils.ML('Er is een fout opgetreden bij het aanvragen van een lock.',
                                       'An error occurred when requesting a lock.');
                    logging.logError('lockFile: ERROR: ' + message);
                }

                var dialogOptions = {
                    'type' : 'info',
                    'title' : utils.ML('Fout bij het locken','Error when locking'),
                    'message' : message,
                    'buttons' : buttons,
                    'defaultID' : 0
                };

                dialog.showMessageBox(dialogOptions,
                    function callback(buttonIndex) {
                        if (buttonIndex === 0) {
                            app.quit();
                        }  else if(buttonIndex === 1) {
                            imsDocument.readonly = true;

                        if(downloaded(imsDocument)) {
                                utils.compareLocalWithRemote(openHTTPFile, imsDocument);
                            } else {
                                openHTTPFile(imsDocument, true);
                            }
                        } else if (buttonIndex === 2) {
                            logging.logVerbose('lockFile: forcing lock 2');
                            lockFile(imsDocument, true);
                            return;
                        }
                    });
            } else {
                console.log(downloaded(imsDocument));
                if(downloaded(imsDocument)) {
                    if(useCached(imsDocument)) {
                        openFile(imsDocument);
                    } else {
                        openHTTPFile(imsDocument, true);
                    }
                } else {
                    openHTTPFile(imsDocument, true);
                }
            }
    });
}

/**
 * Verify the link that is passed as argument to the application. If there is
 * a parameter missing, an error is thrown and the application quits.
 *
 * @param IMSDoc imsDoc - IMSDoc containing a parsed protonlink
 */
function verifyLink(imsDoc)
{
    if(imsDoc.hostName === '' ||
        imsDoc.sessionKey === '' ||
        imsDoc.readonly === '' ||
        imsDoc.docDirectory === '' ||
        imsDoc.fileName  === '' ||
        imsDoc.filePath === '' ||
        imsDoc.imsSiteDirectory === '') {

        var message = utils.ML('Een of meerdere parameters ontbreken', 'One or more parameters are missing');
        dialog.showErrorBox('Fout', message);
        //ericd 100117
        logging.logError('verifyLink: ERROR: '+ message);
        app.quit();
    } else {
        //ericd 100117
        logging.logVerbose('verifyLink: success');

        if(imsDoc.readonly) {
            if(downloaded(imsDoc)) {
                if(useCached(imsDoc)) {
                    openFile(imsDoc);
                } else {
                    utils.compareLocalWithRemote(openHTTPFile, imsDoc);
                }
            } else {
                openHTTPFile(imsDoc, true);
            }
        } else if(!imsDoc.readonly) {
            lockFile(imsDoc, false);
        }
    }
}
