/**.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-****
 *                                                                                                                *
 *       OpenIMS Mac Transfer Agent                                                                               *
 *       Copyright (c) 2001-2020 OpenSesame ICT.                                                                  *
 *       De licentievoorwaarden voor deze software kunt u vinden in /openims/license.txt                          *
 *       of op http://www.openims.com/openims/license.txt                                                         *
 *                                                                                                                *
 *       This header is in Dutch. Use of the OpenIMS Mac Transfer Agent software without respecting               *
 *       the terms of use is prohibited. Copyright (C) 2001-2020 OpenSesame ICT.                                  *
 *       All Rights Reserved.                                                                                     *
 *                                                                                                                *
 ***.-*.-..*.-..***-.--*---*..-*.-.***-...*.-*...*.***.-*.-.*.***-...*.*.-..*---*-.*--.***-*---***..-*...*.-.-.-**/
 /**
 * @file uploadfile.js
 * @brief Provides functions for uploading files to OpenIMS
 * @author Michiel Westerink
 */
const {app} = require('electron');
const {dialog} = require('electron');

const fs = require('fs');
const request = require('request');

const config = require('./config');
const logging = require('./logging');
const utils = require('./utils');

module.exports = {
    uploadFile,
    unlockFile
}

/**
 * Upload a file to OpenIMS.
 *
 * @param IMSDoc imsDoc - IMSDocument object
 * @param [optional] bool changed - True if the file has been changed, false
 * if not
 */
function uploadFile(imsDoc, changed)
{

    logging.logVerbose('uploadFile: changed: ' + changed);
    if(!changed) {
        unlockFile(imsDoc);
    } else {  //ericd 011216 else added

        var filePath = utils.getApplicationDirectory() + imsDoc.docDirectory +
            "/" + imsDoc.fileName;

        //ericd 060117 file too large to save: unlock and quit
        //error in buffer.toString for large files (max 192Mb)
        var stats = fs.statSync(filePath);
        var fileSizeInBytes = stats["size"];
        var fileSizeInMegabytes = fileSizeInBytes / 1048576.0;  // 20210201 KVD SD12265 Use Real Megabytes instead of million
        var configuration = config.readConfigFile();
        var maxFileSizeInMegabytes = configuration["maxFileSizeInMegabytes"];
    logging.logWarning(configuration["maxFileSizeInMegabytes"]);

        if(maxFileSizeInMegabytes - fileSizeInMegabytes < 0) {

            //ericd sizes in Proton and OpenIMS differ, show max size
            message = utils.ML("Dit bestand is te groot om op de server te bewaren (maximale grootte: ",
                               "This file is too large to be uploaded to the server (maximum size: ")
                + maxFileSizeInMegabytes.toString() + "MB).\n\r"+
                utils.ML("Het lokale bestand met de wijzigingen staat hier:\n\r",
                         "The local file with changes is here:\n\r")
                + filePath;

            var buttons = ['OK'];
            var dialogOptions = {
               'type' : 'info',
               'title' : 'dialog',
               'message' : message,
               'buttons' : buttons
            }
            dialog.showMessageBox(dialogOptions,
                   function callback(buttonIndex) {
                       if(buttonIndex === 0) {
                            unlockFile(imsDoc);
                            app.quit();
                       }
                   });

        } else {

            //utils.showDialog(fileSizeInBytes.toString());

    	    var fileData = fs.readFileSync(filePath);
            fileData = fileData.toString('base64');
            var sessionKey = imsDoc.sessionKey;

            var imsDirectoryArray = imsDoc.httpLink.split("/");

            imsDirectory = imsDirectoryArray.slice(3, imsDirectoryArray.length - 1);
            imsDirectory = '/' + imsDirectory.join("/");

    	    var formData = {
    		    sessionkey : sessionKey,
    		    command : 'openims_put3',
    		    file : imsDirectory + '/' + imsDoc.fileName,
    		    content : fileData
    	    };

    	    request.post({url: imsDoc.https + '://' + imsDoc.hostName +
                '/nkit/gate.php',
                formData: formData},
    		    function optionalCallback(err, response, body) {
    			    if (err) {
    				    logging.logVerbose('uploadFile: ERROR: ' + err);
    			    }
    			    if (response) {
    				    logging.logVerbose('uploadFile: response: ' + response);
    			    }
    			    if (body) {
    				    logging.logVerbose('uploadFile: body: ' + body);
    			    }

                    utils.compareLocalWithRemote(signalOpenIMS, imsDoc);
    		    }
            );
        }
    }
}

/**
 * Update the last edited date of a file after uploading to OpenIMS
 *
 * @param IMSDoc imsDocument - IMSDocument to signal
 * @param bool changed - True if the MD5 hash of the local document differs
 * from the MD5 hash of the document on the server, false if not.
 */
function signalOpenIMS(imsDocument, changed)
{
    //logging.logVerbose('signal function');
    var imsDirectoryArray = imsDocument.httpLink.split("/");
    logging.logVerbose('signalOpenIMS: changed: ' + changed);

    //Check if the MD5 of the file on disk differs from the file on the server.
    //If there is a difference, something went wrong with the upload
    if(!changed) {
        imsDirectory = imsDirectoryArray.slice(3, imsDirectoryArray.length - 1);
        imsDirectory = '/' + imsDirectory.join("/");

        var formData = {
            'sessionkey' : imsDocument.sessionKey,
            'command' : 'openims_signal',
            'file' : imsDirectory + '/' + imsDocument.fileName
        };

        request.post({url: imsDocument.https + '://' + imsDocument.hostName +
            '/nkit/gate.php',
            formData: formData}, function callback(err, response, body) {
                if(err) {
                    logging.logVerbose('signalOpenIMS: ERROR: ' + err);
                } else {
                    unlockFile(imsDocument);
                }
        });
    } else if(changed) { //Verification failed, ask user to try again or quit
        var buttons = [utils.ML('Afsluiten','Close'), utils.ML('Opnieuw', 'Try gain')];

        var dialogOptions = {
            'type' : 'info',
            'title' : utils.ML('Uploaden mislukt','Upload failed'),
            'message' : utils.ML('Verificatie na het uploaden is mislukt','Verification after upload has failed'),
            'buttons' : buttons,
            'defaultID' : 1
        };

        dialog.showMessageBox(dialogOptions,
            function callback(buttonIndex) {
                if(buttonIndex === 1) {
                    uploadFile(imsDocument, changed);
                } else  if(buttonIndex === 0) {
                    unlockFile(imsDocument);
                }
            });
    }
}

/**
 * Unlock the IMSDocument after the user is done with it
 *
 * @param IMSDocument imsDocument - Document to unlock
 */
function unlockFile(imsDocument)
{
    var imsDirectoryArray = imsDocument.httpLink.split("/");

    imsDirectory = imsDirectoryArray.slice(3, imsDirectoryArray.length - 1);
    imsDirectory = '/' + imsDirectory.join("/");

    var formData = {
        'command' : 'openims_unlock',
        'dir' : imsDocument.imsSiteDirectory + imsDocument.fileName,
        'sessionkey' : imsDocument.sessionKey
    };

    request.post({url: imsDocument.https + '://' + imsDocument.hostName +
        '/nkit/gate.php',
        formData: formData}, function callback(err, response, body) {
            if(err) {
                logging.logVerbose('unlockFile: ERROR: ' + err);
            }
            utils.cleanUp(imsDocument);
    });
}
